@@ -48,6 +48,7 @@ import curses
 import math
 import re
 import time
+import functools
 
 import struct
 if platform.system() != 'Windows':
@@ -116,18 +117,31 @@ def calculate_screen_lines(tokens, width, cursor=0):
             pos %= width
     return lines
 
+def forward_if_not_current(func):
+    @functools.wraps(func)
+    def newfunc(self, *args, **kwargs):
+        dest = self.get_dest()
+        if self is dest:
+            return func(self, *args, **kwargs)
+        else:
+            return getattr(self.get_dest(), newfunc.__name__)(*args, **kwargs)
+    return newfunc
+
 
 class FakeStream(object):
     """Provide a fake file object which calls functions on the interface
     provided."""
 
-    def __init__(self, interface):
+    def __init__(self, interface, get_dest):
         self.encoding = getpreferredencoding()
         self.interface = interface
+        self.get_dest = get_dest
 
+    @forward_if_not_current
     def write(self, s):
         self.interface.write(s)
 
+    @forward_if_not_current
     def writelines(self, l):
         for s in l:
             self.write(s)
@@ -136,6 +150,10 @@ class FakeStream(object):
         # some third party (amongst them mercurial) depend on this
         return True
 
+    def flush(self):
+        self.interface.flush()
+
+
 class FakeStdin(object):
     """Provide a fake stdin type for things like raw_input() etc."""
 
@@ -162,6 +180,9 @@ class FakeStdin(object):
     def isatty(self):
         return True
 
+    def fileno(self):
+        return 0
+
     def readline(self, size=-1):
         """I can't think of any reason why anything other than readline would
         be useful in the context of an interactive interpreter so this is the
@@ -491,15 +512,18 @@ class CLIRepl(repl.Repl):
         # isn't at the end of the line, but that's what this does for now.
         if self.cpos: return
 
-        # look from right to left for a bad method character
+        # look from right to left for a bad method or dictionary character
         l = len(self.s)
         is_method_char = lambda c: c.isalnum() or c in ('.', '_')
+        dict_chars = ['[']
 
-        if not self.s or not is_method_char(self.s[l-1]):
+        if not self.s or not (is_method_char(self.s[-1])
+                                or self.s[-1] in dict_chars):
             return
 
         for i in range(1, l+1):
-            if not is_method_char(self.s[-i]):
+            c = self.s[-i]
+            if not (is_method_char(c) or c in dict_chars):
                 i -= 1
                 break
 
@@ -995,6 +1019,9 @@ class CLIRepl(repl.Repl):
                 self.do_exit = True
                 return None
 
+        elif key == '\x18':
+            return self.send_current_line_to_editor()
+
         elif key[0:3] == 'PAD' and not key in ('PAD0', 'PADSTOP'):
             pad_keys = {
                 'PADMINUS': '-',
@@ -1108,10 +1135,10 @@ class CLIRepl(repl.Repl):
         self.push('from bpython._internal import _help as help\n', False)
 
         self.iy, self.ix = self.scr.getyx()
-        more = False
+        self.more = False
         while not self.do_exit:
             self.f_string = ''
-            self.prompt(more)
+            self.prompt(self.more)
             try:
                 inp = self.get_line()
             except KeyboardInterrupt:
@@ -1132,8 +1159,8 @@ class CLIRepl(repl.Repl):
             else:
                 self.stdout_hist += inp.encode(getpreferredencoding()) + '\n'
             stdout_position = len(self.stdout_hist)
-            more = self.push(inp)
-            if not more:
+            self.more = self.push(inp)
+            if not self.more:
                 self.prev_block_finished = stdout_position
                 self.s = ''
         return self.exit_value
@@ -1203,8 +1230,8 @@ class CLIRepl(repl.Repl):
             # I decided it was easier to just do this manually
             # than to make the print_line and history stuff more flexible.
             self.scr.addstr('\n')
-            more = self.push(line)
-            self.prompt(more)
+            self.more = self.push(line)
+            self.prompt(self.more)
             self.iy, self.ix = self.scr.getyx()
 
         self.cpos = 0
@@ -1257,15 +1284,22 @@ class CLIRepl(repl.Repl):
             max_h = y + 1
         max_w = int(w * self.config.cli_suggestion_width)
         self.list_win.erase()
+
         if items:
             sep = '.'
-            if os.path.sep in items[0]:
-                # Filename completion
-                sep = os.path.sep
-            if sep in items[0]:
-                items = [x.rstrip(sep).rsplit(sep)[-1] for x in items]
+            separators = ['.', os.path.sep, '[']
+            lastindex = max([items[0].rfind(c) for c in separators])
+            if lastindex > -1:
+                sep = items[0][lastindex]
+            items = [x.rstrip(sep).rsplit(sep)[-1] for x in items]
+            if current_item:
+                current_item = current_item.rstrip(sep).rsplit(sep)[-1]
+
+            if items[0].endswith(']'):
+                # dictionary key suggestions
+                items = [x.rstrip(']') for x in items]
                 if current_item:
-                    current_item = current_item.rstrip(sep).rsplit(sep)[-1]
+                    current_item = current_item.rstrip(']')
 
         if topline:
             height_offset = self.mkargspec(topline, down) + 1
@@ -1403,7 +1437,7 @@ class CLIRepl(repl.Repl):
             os.kill(os.getpid(), signal.SIGSTOP)
 
     def tab(self, back=False):
-        """Process the tab key being hit. 
+        """Process the tab key being hit.
 
         If there's only whitespace
         in the line or the line is blank then process a normal tab,
@@ -1505,6 +1539,46 @@ class CLIRepl(repl.Repl):
         self.addstr(self.cut_buffer)
         self.print_line(self.s, clr=True)
 
+    def send_current_line_to_editor(self):
+        lines = self.send_to_external_editor(self.s).split('\n')
+        self.s = ''
+        self.print_line(self.s)
+        while lines and not lines[-1]:
+            lines.pop()
+        if not lines:
+            return ''
+
+        self.f_string = ''
+        self.cpos = -1 # Set cursor position to -1 to prevent paren matching
+
+        self.iy, self.ix = self.scr.getyx()
+        self.evaluating = True
+        for line in lines:
+            if py3:
+                self.stdout_hist += line + '\n'
+            else:
+                self.stdout_hist += line.encode(getpreferredencoding()) + '\n'
+            self.history.append(line)
+            self.print_line(line)
+            self.s_hist[-1] += self.f_string
+            self.scr.addstr('\n')
+            self.more = self.push(line)
+            self.prompt(self.more)
+            self.iy, self.ix = self.scr.getyx()
+        self.evaluating = False
+
+        self.cpos = 0
+        indent = repl.next_indentation(self.s, self.config.tab_length)
+        self.s = ''
+        self.scr.refresh()
+
+        if self.buffer:
+            for _ in xrange(indent):
+                self.tab()
+
+        self.print_line(self.s)
+        self.scr.redrawwin()
+        return ''
 
 class Statusbar(object):
     """This class provides the status bar at the bottom of the screen.
@@ -1616,7 +1690,7 @@ class Statusbar(object):
                 curses.flushinp()
                 raise ValueError
             # literal
-            elif 0 <= c < 127:
+            elif 0 < c < 127:
                 c = chr(c)
                 self.win.addstr(c, get_colpair(self.config, 'prompt'))
                 o += c
@@ -1773,7 +1847,10 @@ def do_resize(caller):
     curses.doupdate()
     DO_RESIZE = False
 
-    caller.resize()
+    try:
+        caller.resize()
+    except curses.error:
+        pass
     # The list win resizes itself every time it appears so no need to do it here.
 
 
@@ -1870,14 +1947,21 @@ def main_curses(scr, args, config, interactive=True, locals_=None,
     clirepl._C = cols
 
     sys.stdin = FakeStdin(clirepl)
-    sys.stdout = FakeStream(clirepl)
-    sys.stderr = FakeStream(clirepl)
+    sys.stdout = FakeStream(clirepl, lambda: sys.stdout)
+    sys.stderr = FakeStream(clirepl, lambda: sys.stderr)
 
     if args:
-        bpython.args.exec_code(interpreter, args)
+        exit_value = ()
+        try:
+            bpython.args.exec_code(interpreter, args)
+        except SystemExit, e:
+            # The documentation of code.InteractiveInterpreter.runcode claims
+            # that it reraises SystemExit. However, I can't manage to trigger
+            # that. To be one the safe side let's catch SystemExit here anyway.
+            exit_value = e.args
         if not interactive:
             curses.raw(False)
-            return clirepl.getstdout()
+            return (exit_value, clirepl.getstdout())
     else:
         sys.path.insert(0, '')
         clirepl.startup()
@@ -1886,6 +1970,9 @@ def main_curses(scr, args, config, interactive=True, locals_=None,
         clirepl.write(banner)
         clirepl.write('\n')
     exit_value = clirepl.repl()
+    if hasattr(sys, 'exitfunc'):
+        sys.exitfunc()
+        delattr(sys, 'exitfunc')
 
     main_win.erase()
     main_win.refresh()
