@@ -53,6 +53,10 @@ def complete(line, cw):
     if not cw:
         return None
 
+    # TODO if this is done in a thread (as it prob will be in Windows) we'll need this
+    # if not fully_loaded:
+    #     return []
+
     tokens = line.split()
     if tokens[0] not in ['from', 'import']:
         return None
@@ -134,7 +138,7 @@ def find_modules(path):
             with catch_warnings():
                 warnings.simplefilter("ignore", ImportWarning)
                 fo, pathname, _ = imp.find_module(name, [path])
-        except (ImportError, SyntaxError):
+        except (ImportError, IOError, SyntaxError):
             continue
         except UnicodeEncodeError:
             # Happens with Python 3 when there is a filename in some
