@@ -28,8 +28,10 @@ import errno
 import inspect
 import os
 import pydoc
+import shlex
 import subprocess
 import sys
+import tempfile
 import textwrap
 import traceback
 import unicodedata
@@ -386,8 +388,6 @@ class Repl(object):
         self._C = {}
         self.prev_block_finished = 0
         self.interact = Interaction(self.config)
-        self.ps1 = '>>> '
-        self.ps2 = '... '
         # previous pastebin content to prevent duplicate pastes, filled on call
         # to repl.pastebin
         self.prev_pastebin_content = ''
@@ -401,6 +401,20 @@ class Repl(object):
             self.rl_history.load(pythonhist,
                     getpreferredencoding() or "ascii")
 
+    @property
+    def ps1(self):
+        try:
+            return str(sys.ps1)
+        except AttributeError:
+            return '>>> '
+
+    @property
+    def ps2(self):
+        try:
+            return str(sys.ps2)
+        except AttributeError:
+            return '... '
+
     def startup(self):
         """
         Execute PYTHONSTARTUP file if it exits. Call this after front
@@ -704,17 +718,32 @@ class Repl(object):
 
         if fn.startswith('~'):
             fn = os.path.expanduser(fn)
+        if not fn.endswith('.py') and self.config.save_append_py:
+            fn = fn + '.py'
+
+        mode = 'w'
+        if os.path.exists(fn):
+            mode = self.interact.file_prompt('%s already exists. Do you want '
+                                             'to (c)ancel, (o)verwrite or '
+                                             '(a)ppend? ' % (fn, ))
+            if mode in ('o', 'overwrite'):
+                mode = 'w'
+            elif mode in ('a', 'append'):
+                mode = 'a'
+            else:
+                self.interact.notify('Save cancelled.')
+                return
 
         s = self.formatforfile(self.getstdout())
 
         try:
-            f = open(fn, 'w')
+            f = open(fn, mode)
             f.write(s)
             f.close()
         except IOError:
             self.interact.notify("Disk write error for file '%s'." % (fn, ))
         else:
-            self.interact.notify('Saved to %s' % (fn, ))
+            self.interact.notify('Saved to %s.' % (fn, ))
 
     def pastebin(self, s=None):
         """Upload to a pastebin and display the URL in the status bar."""
@@ -817,21 +846,7 @@ class Repl(object):
         self.buffer.append(s)
 
         if insert_into_history:
-            if self.config.hist_length:
-                histfilename = os.path.expanduser(self.config.hist_file)
-                oldhistory = self.rl_history.entries
-                self.rl_history.entries = []
-                if os.path.exists(histfilename):
-                    self.rl_history.load(histfilename, getpreferredencoding())
-                self.rl_history.append(s)
-                try:
-                    self.rl_history.save(histfilename, getpreferredencoding(), self.config.hist_length)
-                except EnvironmentError, err:
-                    self.interact.notify("Error occured while writing to file %s (%s) " % (histfilename, err.strerror))
-                    self.rl_history.entries = oldhistory
-                    self.rl_history.append(s)
-            else:
-                self.rl_history.append(s)
+            self.insert_into_history(s)
 
         more = self.interp.runsource('\n'.join(self.buffer))
 
@@ -840,6 +855,23 @@ class Repl(object):
 
         return more
 
+    def insert_into_history(self, s):
+        if self.config.hist_length:
+            histfilename = os.path.expanduser(self.config.hist_file)
+            oldhistory = self.rl_history.entries
+            self.rl_history.entries = []
+            if os.path.exists(histfilename):
+                self.rl_history.load(histfilename, getpreferredencoding())
+            self.rl_history.append(s)
+            try:
+                self.rl_history.save(histfilename, getpreferredencoding(), self.config.hist_length)
+            except EnvironmentError, err:
+                self.interact.notify("Error occured while writing to file %s (%s) " % (histfilename, err.strerror))
+                self.rl_history.entries = oldhistory
+                self.rl_history.append(s)
+        else:
+            self.rl_history.append(s)
+
     def undo(self, n=1):
         """Go back in the undo history n steps and call reeavluate()
         Note that in the program this is called "Rewind" because I
@@ -877,7 +909,15 @@ class Repl(object):
         """See the flush() method docstring."""
 
     def tokenize(self, s, newline=False):
-        """Tokenize a line of code."""
+        """Tokenizes a line of code, returning pygments tokens
+        with side effects/impurities:
+        - reads self.cpos to see what parens should be highlighted
+        - reads self.buffer to see what came before the passed in line
+        - sets self.highlighted_paren to (buffer_lineno, tokens_for_that_line) for buffer line
+            that should replace that line to unhighlight it
+        - calls reprint_line with a buffer's line's tokens and the buffer lineno that has changed
+            iff that line is the not the current line
+        """
 
         source = '\n'.join(self.buffer + [s])
         cursor = len(source) - self.cpos
@@ -966,6 +1006,18 @@ class Repl(object):
         """This is used as the exception callback for the Interpreter instance.
         It prevents autoindentation from occuring after a traceback."""
 
+    def send_to_external_editor(self, text, filename=None):
+        """Returns modified text from an editor, or the oriignal text if editor exited with non-zero"""
+        editor_args = shlex.split(self.config.editor)
+        with tempfile.NamedTemporaryFile(suffix='.py') as temp:
+            temp.write(text)
+            temp.flush()
+            if subprocess.call(editor_args + [temp.name]) == 0:
+                with open(temp.name) as f:
+                    return f.read()
+            else:
+                return text
+
 
 def next_indentation(line, tab_length):
     """Given a code line, return the indentation of the next line."""
@@ -1038,3 +1090,4 @@ def extract_exit_value(args):
         return args[0]
     else:
         return args
+
